
package com.perforce.client.api;

import java.io.*;

/**
 * Perforce file I/O.
 * 
 * The FileSys class provides a platform-independent set of methods used to 
 * create, read and write files to disk.
 * 
 * <p> You can intercept the file I/O and implement your own client workspace 
 * file access routines by replacing {@link ClientUser#file} in a ClientUser 
 * subclass.
 * 
 * <dl><dt> <b>Note</b>
 * <dd> <hr> Replacing the existing I/O routines is non-trivial. Your replacement routines 
 * must handle all special cases, including cross-platform file issues.
 * 
 * <dd> Unless your application has highly specialized requirements, (for instance, 
 * performing all file I/O in memory rather than on disk), this approach is not recommended.
 * <hr></dl>
 * 
 * <p> If you intend to replace {@link ClientUser#file file()}, all of the virtual methods 
 * documented are required. The non virtual methods are not required and not documented. 
 */
public class FileSys
{
    private boolean javaCreatedTheInstance;
    long instance = 0;
    
//  private static native long nNewInstance();
    private static native void nDeleteInstance(long instance);
    
    private File javaFile = null;
    
/*
    FileSys()
    {
        this.instance = nNewInstance();
        javaCreatedTheInstance = true;
    }
*/  
    FileSys(long instance)
    {
        this.instance = instance;
        javaCreatedTheInstance = false;
    }
    
    protected void finalize() throws Throwable {
        if (javaCreatedTheInstance) {
            nDeleteInstance(this.instance);
        }
        super.finalize();
    }
    
    private static native String nName(long instance);
    
    private File iGetFile()
    {
        if (javaFile == null)
        {
            javaFile = new File(nName(this.instance));
        }
        
        return javaFile;
    }
    
    public File getJavaFile()
    {
        return iGetFile();
    }
    
}